/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Super 'Wait' Application
 * 
 * Copyright (C) 1999, Mark Spencer
 *
 * Mark Spencer <markster@linux-support.net>
 *
 * Contributed By Christopher Wade <clwade@sparco.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/options.h>
#include <asterisk/musiconhold.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/lock.h>
#include <asterisk/utils.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

static char *app = "SWait";
static char *synopsis = "Super 'Wait' Application";
static char *tdesc = 
"SWait([timeout][cim])\n"
"  Simply put, combines Wait, WaitExten, and WaitMusicOnHold into one, simple,\n"
"powerful, easy to use function.  Timeout defaults to 'ResponseTimeout'.  'c'\n"
"makes SWait continue at pri+1 instead of doing a Goto(t,1).  'i' makes SWait\n"
"ignore DTMF input.  'm' makes SWait play moh instead of silence.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int swait_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;

	LOCAL_USER_ADD(u);

	int ms;
	char* wait;
	char* options;
	int option_continue = 0;
	int option_moh = 0;
	int option_ignoredtmf = 0;
	double delay = 0;

	if (data && !ast_strlen_zero(data)) {
		if (!(wait = ast_strdupa(data))) {
			ast_log(LOG_WARNING, "Unable to duplicate string :(\n");
		} else {
			delay = strtod(wait, &options);

			while (!ast_strlen_zero(options)) {
				switch (*options) {
					case 'c':
						option_continue++;
						break;
					case 'm':
						option_moh++;
						break;
					case 'i':
						option_ignoredtmf++;
						break;
				}
				options++;
			}
		}
	}

	if (data && delay)
		ms = delay * 1000;
	else if (chan->pbx)
		ms = chan->pbx->rtimeout * 1000;
	else
		ms = 10000;

	if (option_moh && ast_moh_start(chan, NULL)) {
                ast_log(LOG_WARNING, "Unable to start music on hold on channel %s\n", chan->name);
                return -1;
        }

	if (option_ignoredtmf) {
		res = ast_safe_sleep(chan, ms);
	} else {
		res = ast_waitfordigit(chan, ms);
	}

	if (option_moh)
	        ast_moh_stop(chan);

	if (option_continue == 0) {
		if (!res) {
			if (ast_exists_extension(chan, chan->context, "t", 1, chan->cid.cid_num)) {
				if (option_verbose > 2)
					ast_verbose(VERBOSE_PREFIX_3 "Timeout on %s\n", chan->name);
				strncpy(chan->exten, "t", sizeof(chan->exten));
				chan->priority = 0;
			} else {
				ast_log(LOG_WARNING, "Timeout but no rule 't' in context '%s'\n", chan->context);
				res = -1;
			}
		}
	}

	LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, swait_exec, synopsis, tdesc);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
